
/**************************************************************************************
* This file is part of the GameFramework project released by Wazoo Enterprises Inc.
* under the GNU Public License.
*
* Any projects using this code must attach these copyright notices along with their
* own as outlined at http://www.gnu.org
*
* Copyright 2003 by Wazoo Enterprises Inc.
* To contact the auther, send a message to wazoo@wazooenterprises.com
*
****************************************************************************************/

#pragma warning(disable: 4786)

#include "D3DRenderer.h"
#include "D3DVertexBuffer.h"
#include "D3DTextureManager.h"

D3DRenderer::D3DRenderer()
{
	m_lpD3D = NULL;
	m_lpD3DDevice = NULL;

	m_pVB = NULL;

	m_bUseStencilBit = FALSE;
	m_bUseDepthBit   = TRUE;

}

D3DRenderer::~D3DRenderer()
{
	destroyDevice();

}

HRESULT D3DRenderer::createDevice(HWND hWnd,
								  DWORD dwWidth,
								  DWORD dwHeight,
								  DWORD dwDepth,
								  BOOL bWindowed)
{
	HRESULT hr;
	D3DDISPLAYMODE d3ddm;

	m_dwWidth = dwWidth;
	m_dwHeight = dwHeight;
	m_dwDepth = dwDepth;
	m_bWindowed = bWindowed;
	m_hWnd = hWnd;

	m_lpD3D = Direct3DCreate8( D3D_SDK_VERSION );
	if(m_lpD3D == NULL){
		return E_FAIL;
	}

	//get adapter settings
	hr = m_lpD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT,
										&d3ddm);
	if(FAILED(hr)){
		return E_FAIL;
	}

	//For now just create a windowed direct3D device
	ZeroMemory(&m_d3dpp, sizeof(m_d3dpp));
	m_d3dpp.hDeviceWindow = m_hWnd;
	m_d3dpp.BackBufferWidth = m_dwWidth;
	m_d3dpp.BackBufferHeight = m_dwHeight;
	m_d3dpp.BackBufferFormat = d3ddm.Format;
	m_d3dpp.BackBufferCount = 1;
	m_d3dpp.SwapEffect = D3DSWAPEFFECT_FLIP;
	m_d3dpp.MultiSampleType = D3DMULTISAMPLE_NONE;
	m_d3dpp.EnableAutoDepthStencil = TRUE;
	m_d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
	m_d3dpp.Windowed = TRUE;

	//create
	hr = m_lpD3D->CreateDevice(D3DADAPTER_DEFAULT,
							   D3DDEVTYPE_HAL,
							   m_hWnd,
							   D3DCREATE_SOFTWARE_VERTEXPROCESSING,
							   &m_d3dpp,
							   &m_lpD3DDevice);
	if(FAILED(hr)){

		m_d3dpp.AutoDepthStencilFormat = D3DFMT_D32;
		hr = m_lpD3D->CreateDevice(D3DADAPTER_DEFAULT,
							   D3DDEVTYPE_HAL,
							   m_hWnd,
							   D3DCREATE_SOFTWARE_VERTEXPROCESSING,
							   &m_d3dpp,
							   &m_lpD3DDevice);
		if(FAILED(hr)){
			return E_FAIL;
		}
	}

	//Version1.1
	//We need to set our projection matrix, as well as create our vertexbuffer
	//pointer

	// Set the projection matrix
    D3DXMATRIX matProj;
    FLOAT fAspect = (FLOAT)m_dwWidth / m_dwHeight;

	//Don't forget that usually Direct3D works with the left handed coordinate
	//system, whereas the OpenGL pipeline works with the right handed one. In order
	//to maintain compatibility with the OpenGL interface, all of our projection
	//and view matrix calculations need to be converted to a Right Handed matrix
	D3DXMatrixPerspectiveFovRH( &matProj, D3DX_PI/4, fAspect, 1.0f, 100.0f );
    m_lpD3DDevice->SetTransform( D3DTS_PROJECTION, &matProj );

	m_lpD3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);


	//create our vertex buffer interface pointer. For now we're just gonna pick
	//an arbitrary value of 1000 vertices for our buffer size, though we SHOULD
	//be checking the D3D8CAPS for our device and going from there...
	m_pVB = new D3DVertexBuffer();
	if(FAILED(hr = m_pVB->createVB(1000, m_lpD3DDevice))){
		return E_FAIL;
	}

	//create our texture manager interface
	m_pTextureManager = new D3DTextureManager(m_lpD3DDevice);



	return S_OK;
}

void D3DRenderer::destroyDevice(void)
{

	if(m_pTextureManager){
		delete m_pTextureManager;
		m_pTextureManager = NULL;
	}

	if(m_pVB){
		delete m_pVB;
		m_pVB = NULL;
	}

	if(m_lpD3DDevice)
	{
		m_lpD3DDevice->Release();
		m_lpD3DDevice = NULL;
	}

	if(m_lpD3D)
	{
		m_lpD3D->Release();
		m_lpD3D = NULL;
	}

}

HRESULT D3DRenderer::beginRenderingScene(void)
{
	DWORD dwClearParam = D3DCLEAR_TARGET;

	if(m_bUseDepthBit){
		dwClearParam = dwClearParam | D3DCLEAR_ZBUFFER;
	}

	if(m_bUseStencilBit){
		dwClearParam = dwClearParam | D3DCLEAR_STENCIL;
	}

	m_lpD3DDevice->Clear(0,
					   NULL,
					   dwClearParam,
					   m_backColor,
					   1.0f,
					   0);

	return(m_lpD3DDevice->BeginScene() );

}

void D3DRenderer::endRenderingScene(void)
{

	m_lpD3DDevice->EndScene();
	m_lpD3DDevice->Present(NULL, NULL, NULL, NULL);

}

void D3DRenderer::setClearColor(FLOAT fRed,
								FLOAT fGreen,
								FLOAT fBlue,
								FLOAT fAlpha)
{
	m_backColor = D3DCOLOR_COLORVALUE(fRed, fGreen, fBlue, fAlpha);


}

void D3DRenderer::setState( const eState STATE, const eOp OP ){

	switch(STATE)
	{
	case LIGHTING:
		if(OP == ENABLE){
			m_lpD3DDevice->SetRenderState( D3DRS_LIGHTING, TRUE );
		}else if( OP == DISABLE){
			m_lpD3DDevice->SetRenderState( D3DRS_LIGHTING, FALSE );
		}
		break;

	case ZBUFFER:
		if(OP == ENABLE){
			m_lpD3DDevice->SetRenderState( D3DRS_ZENABLE, D3DZB_TRUE );
		}else if(OP == DISABLE){
			m_lpD3DDevice->SetRenderState( D3DRS_ZENABLE, D3DZB_FALSE );
		}
		break;

	case ALPHABLENDING:
		if(OP == ENABLE) {
			m_lpD3DDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );
		}else if(OP == DISABLE){
			m_lpD3DDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, FALSE );
		}
		break;

	case SRC_BLEND:
		if(OP == SRC_ALPHA){
			m_lpD3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA );
		}

		break;

	case DEST_BLEND:
		if(OP == INV_SRC_ALPHA){
			m_lpD3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
		}

		break;

	};

}

void D3DRenderer::setFunction(const eFunction FUNCTION, const eCompareOp OP, FLOAT fRef)
{
	DWORD dwRef = FtoDW(fRef);

	switch(FUNCTION)
	{
	case ALPHATEST_FUNC:
		switch(OP)
		{
			case NEVER:
				m_lpD3DDevice->SetRenderState( D3DRS_ALPHAFUNC, D3DCMP_NEVER);
			break;
			case ALWAYS:
				m_lpD3DDevice->SetRenderState( D3DRS_ALPHAFUNC, D3DCMP_ALWAYS);
			break;
			case EQUAL:
				m_lpD3DDevice->SetRenderState( D3DRS_ALPHAFUNC, D3DCMP_EQUAL);
			break;
			case GREATEREQUAL:
				m_lpD3DDevice->SetRenderState( D3DRS_ALPHAFUNC, D3DCMP_GREATEREQUAL);
			break;
		};

		m_lpD3DDevice->SetRenderState( D3DRS_ALPHAREF, dwRef );

		break;
	case ZBUFFER_FUNC:
		switch(OP)
		{
			case NEVER:
				m_lpD3DDevice->SetRenderState( D3DRS_ZFUNC, D3DCMP_NEVER);
			break;
			case ALWAYS:
				m_lpD3DDevice->SetRenderState( D3DRS_ZFUNC, D3DCMP_ALWAYS);
			break;
			case EQUAL:
				m_lpD3DDevice->SetRenderState( D3DRS_ZFUNC, D3DCMP_EQUAL);
			break;
			case GREATEREQUAL:
				m_lpD3DDevice->SetRenderState( D3DRS_ZFUNC, D3DCMP_GREATEREQUAL);
			break;
		}

		break;

	case STENCIL_FUNC:
		switch(OP)
		{
			case NEVER:
				m_lpD3DDevice->SetRenderState( D3DRS_STENCILFUNC, D3DCMP_NEVER);
			break;
			case ALWAYS:
				m_lpD3DDevice->SetRenderState( D3DRS_STENCILFUNC, D3DCMP_ALWAYS);
			break;
			case EQUAL:
				m_lpD3DDevice->SetRenderState( D3DRS_STENCILFUNC, D3DCMP_EQUAL);
			break;
			case GREATEREQUAL:
				m_lpD3DDevice->SetRenderState( D3DRS_STENCILFUNC, D3DCMP_GREATEREQUAL);
			break;
		};

		m_lpD3DDevice->SetRenderState( D3DRS_STENCILREF, dwRef );

		break;
	};

}
