
/**************************************************************************************
* This file is part of the GameFramework project released by Wazoo Enterprises Inc.
* under the GNU Public License. 
*
* Any projects using this code must attach these copyright notices along with their
* own as outlined at http://www.gnu.org
*
* Copyright 2003 by Wazoo Enterprises Inc.
* To contact the auther, send a message to wazoo@wazooenterprises.com
*
****************************************************************************************/


//Here's our OpenGL creation/destruction code. Note
//that I got a lot of help getting OpenGL up and running
//on the win32 platform with the help of nehe's tutorial
//site. http://nehe.gamedev.net


#pragma warning(disable: 4786)

#include "OGLRenderer.h"
#include "OGLVertexBuffer.h"
#include "OGLTextureManager.h"

OGLRenderer::OGLRenderer()
{
	m_hDC = NULL;
	m_hRC = NULL;

	m_pVB = NULL;
	m_pTextureManager = NULL;

	m_srcBlendOp = GL_ONE;
	m_destBlendOp = GL_ZERO;

	m_bUseStencilBit = FALSE;
	m_bUseDepthBit   = TRUE;

}

OGLRenderer::~OGLRenderer()
{
	destroyDevice();

}

HRESULT OGLRenderer::createDevice(HWND hWnd,
								  DWORD dwWidth,
								  DWORD dwHeight,
								  DWORD dwDepth,
								  BOOL bWindowed)
{
	HRESULT hr;

	m_dwWidth = dwWidth;
	m_dwHeight = dwHeight;
	m_dwDepth = dwDepth;
	m_bWindowed = bWindowed;
	m_hWnd = hWnd;

	
	static PIXELFORMATDESCRIPTOR pfd =					// pfd Tells Windows How We Want Things To Be
	{
		sizeof(PIXELFORMATDESCRIPTOR),					// Size Of This Pixel Format Descriptor
		1,								// Version Number
		PFD_DRAW_TO_WINDOW |						// Format Must Support Window
		PFD_SUPPORT_OPENGL |						// Format Must Support OpenGL
		PFD_DOUBLEBUFFER,						// Must Support Double Buffering
		PFD_TYPE_RGBA,							// Request An RGBA Format
		(int)m_dwDepth,								// Select Our Color Depth
		0, 0, 0, 0, 0, 0,						// Color Bits Ignored
		0,								// No Alpha Buffer
		0,								// Shift Bit Ignored
		0,								// No Accumulation Buffer
		0, 0, 0, 0,							// Accumulation Bits Ignored
		16,								// 16Bit Z-Buffer (Depth Buffer)
		0,								// No Stencil Buffer
		0,								// No Auxiliary Buffer
		PFD_MAIN_PLANE,							// Main Drawing Layer
		0,								// Reserved
		0, 0, 0								// Layer Masks Ignored
	};

	if (!(m_hDC=GetDC(hWnd)))							// Did We Get A Device Context?
	{
		destroyDevice();
		MessageBox(NULL,"Can't Create A GL Device Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return E_FAIL;							// Return FALSE
	}

	GLuint pixelFormat;
	if (!(pixelFormat=ChoosePixelFormat(m_hDC,&pfd))) // Did Windows Find A Matching Pixel Format?
	{
		destroyDevice();							// Reset The Display
		MessageBox(NULL,"Can't Find A Suitable PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return E_FAIL;							// Return FALSE
	}

	if(!SetPixelFormat(m_hDC,pixelFormat,&pfd))	// Are We Able To Set The Pixel Format?
	{
		destroyDevice();							// Reset The Display
		MessageBox(NULL,"Can't Set The PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return E_FAIL;							// Return FALSE
	}

	if (!(m_hRC=wglCreateContext(m_hDC)))	// Are We Able To Get A Rendering Context?
	{
		destroyDevice();							// Reset The Display
		MessageBox(NULL,"Can't Create A GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return E_FAIL;							// Return FALSE
	}

	if(!wglMakeCurrent(m_hDC,m_hRC))	// Try To Activate The Rendering Context
	{
		destroyDevice();							// Reset The Display
		MessageBox(NULL,"Can't Activate The GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return E_FAIL;							// Return FALSE
	}

	//set our background clearing color
	setClearColor(0.0f, 0.0f, 0.0f, 0.5f);

	//create a perspective projection
	glMatrixMode(GL_PROJECTION);						
	glLoadIdentity();							
	gluPerspective(45.0f,(GLfloat)m_dwWidth/(GLfloat)m_dwHeight,1.0f,100.0f);

	//switch to our model matrix stack
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	m_pVB = new OGLVertexBuffer();
	if(FAILED(hr = m_pVB->createVB(1000, NULL))){
		return E_FAIL;
	}

	glClearDepth(1.0f);
	glShadeModel(GL_SMOOTH);


	//v1.3
	//create our texture manager
	m_pTextureManager = new OGLTextureManager();

	return S_OK;
}

void OGLRenderer::destroyDevice(void)
{

	if(m_pTextureManager){
		delete m_pTextureManager;
		m_pTextureManager = NULL;
	}


	if(m_pVB){
		delete m_pVB;
		m_pVB = NULL;
	}

	if(m_hDC){
		wglMakeCurrent(m_hDC, 0);
		if(m_hRC){
			wglDeleteContext(m_hRC);
			m_hRC = NULL;
		}
		ReleaseDC(m_hWnd, m_hDC);
		m_hRC = NULL;
	}

}

HRESULT OGLRenderer::beginRenderingScene(void)
{
	GLbitfield clearParams = GL_COLOR_BUFFER_BIT;

	if(m_bUseDepthBit){
		clearParams = clearParams | GL_DEPTH_BUFFER_BIT;
	}

	if(m_bUseStencilBit){
		clearParams = clearParams | GL_STENCIL_BUFFER_BIT;
	}

	glClear( clearParams );


	return S_OK;
}

void OGLRenderer::endRenderingScene(void)
{
	SwapBuffers(m_hDC);

}

void OGLRenderer::setClearColor(FLOAT fRed,
								FLOAT fGreen,
								FLOAT fBlue,
								FLOAT fAlpha)
{
	glClearColor(fRed, fGreen, fBlue, fAlpha);

}

void OGLRenderer::setState(const eState STATE, const eOp OP){

	switch(STATE)
	{
	case LIGHTING:
		if(OP == ENABLE){
			glEnable(GL_LIGHTING);
		}else if(OP == DISABLE){
			glDisable(GL_LIGHTING);
		}

		break;
	case ZBUFFER:
		if(OP == ENABLE){
			glEnable(GL_DEPTH_TEST);
		}else if( OP == DISABLE) {
			glDisable(GL_DEPTH_TEST);
		}

		break;
	case ALPHABLENDING:
		if(OP == ENABLE){
			glEnable(GL_BLEND);
		}else if( OP == DISABLE ){
			glDisable(GL_BLEND);
		}

		break;
	case SRC_BLEND:
		if(OP == ONE){
			m_srcBlendOp = GL_ONE;
		}else if(OP == ZERO){
			m_srcBlendOp = GL_ZERO;
		}else if(OP == SRC_ALPHA){
			m_srcBlendOp = GL_SRC_ALPHA;
			OutputDebugString("src blend = src alpha\n");
		}else if(OP == INV_SRC_ALPHA){
			m_srcBlendOp = GL_ONE_MINUS_SRC_ALPHA;
		}else if(OP == DEST_ALPHA){
			m_srcBlendOp = GL_DST_ALPHA;
		}else if(OP == INV_DEST_ALPHA){
			m_srcBlendOp = GL_ONE_MINUS_DST_ALPHA;
		}

		glBlendFunc(m_srcBlendOp, m_destBlendOp);

		
		break;
	case DEST_BLEND:
		if(OP == SRC_ALPHA){
			m_destBlendOp = GL_SRC_ALPHA;
		}else if(OP == INV_SRC_ALPHA){
			m_destBlendOp = GL_ONE_MINUS_SRC_ALPHA;
			OutputDebugString("dest blend = one minus src alpha\n");
		}else if(OP == ONE){
			m_destBlendOp = GL_ONE;
		}else if(OP == ZERO){
			m_destBlendOp = GL_ZERO;
		}else if(OP == DEST_ALPHA){
			m_destBlendOp = GL_DST_ALPHA;
		}
		

		glBlendFunc(m_srcBlendOp, m_destBlendOp);

		break;
	case ALPHATEST:
		if(OP == ENABLE){
			glEnable(GL_ALPHA_TEST);
		}else if(OP == DISABLE){
			glDisable(GL_ALPHA_TEST);
		}
		break;

	case STENCIL:
		if(OP == ENABLE){
			glEnable(GL_STENCIL_TEST);
		
		}else if(OP == DISABLE){
			glDisable(GL_STENCIL_TEST);
			
		}
		break;
	};
	
}

void OGLRenderer::setFunction(const eFunction FUNCTION, const eCompareOp OP, FLOAT fRef)
{
	
	switch(FUNCTION)
	{
		case ALPHATEST_FUNC:
			switch(OP)
			{
			case NEVER:
				glAlphaFunc(GL_NEVER, fRef);
				break;
			case ALWAYS:
				glAlphaFunc(GL_ALWAYS, fRef);
				break;

			case EQUAL:
				glAlphaFunc(GL_EQUAL, fRef);
				break;
			case GREATEREQUAL:
				glAlphaFunc(GL_GEQUAL, fRef);
				break;
			};

		break;

		case ZBUFFER_FUNC:

		break;
		case STENCIL_FUNC:

		break;
	};



}



