
#ifndef __OGLVERTEXBUFFER_H_
#define __OGLVERTEXBUFFER_H_

/**************************************************************************************
* This file is part of the GameFramework project released by Wazoo Enterprises Inc.
* under the GNU Public License. 
*
* Any projects using this code must attach these copyright notices along with their
* own as outlined at http://www.gnu.org
*
* Copyright 2003 by Wazoo Enterprises Inc.
* To contact the auther, send a message to wazoo@wazooenterprises.com
*
****************************************************************************************/

#include <windows.h>
#include <d3dx8.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "vbInterface.h"

/**
* This object encapsulates the OpenGL implementation of our
* vbInterface object. Basically it will act as a way to add
* triangle data to our graphics pipeline.
*/
class OGLVertexBuffer : public vbInterface {

public:
	/**
	* Constructor
	*/
	OGLVertexBuffer();

	/**
	* Destructor
	*/
	~OGLVertexBuffer();
		
	/**
	* This method is responsible for creating our vertex buffer interface
	* object. Because OpenGL and Direct3D8 initialize things just a little
	* bit differently, I decided to give us the flexibility to pass in any
	* "extra" object that we would need to create our vertex buffer.
	* @param int - How many triangles our vertex buffer should hold
	* @param VOID* - A pointer to any extra creation object we need
	* @return HRESULT - SUCCEEDED if things are ok, FAILED otherwise
	*/
	HRESULT createVB(int, VOID* = NULL);

	/**
	* This method just destroys our vertex buffer and handles any
	* memory cleanup
	* @param void
	* @return void
	*/
	void destroyVB(void);

	/**
	* This method is responsible for locking down the vertex buffer
	* memory. Basically the hardware will just lock down an area that
	* we can use to add/update any triangle data.
	* @param void
	* @return HRESULT - if we SUCCEEDED or FAILED
	*/
	HRESULT lockVB(void);

	/**
	* This method is responsible for signalling to the vertex buffer
	* that we are finished adding triangles. Let's unlock the video
	* memory, so that we can perform other operations with the 
	* hardware/pipeline
	* @param void
	* @return void
	*/
	void unlockVB(void);

	/**
	* This method is responsible for adding a triangle to our vertex
	* buffer interface. Although it IS a bit of a waste having a virtual
	* method being called for every single triangle that we add, we 
	* are just trying to create a way to demonstrate a fashion of rendering
	* primitives with both OpenGL and Direct3D8. The reader can optimize
	* as desired.
	* @param D3DXVECTOR3 - (x, y, z) position of our vertex
	* @param FLOAT       - red component of our vertex
	* @param FLOAT       - green component of our vertex
	* @param FLOAT       - blue component of our vertex
	* @param FLOAT       - alpha component of our vertex
	* @return void 
	*/
	void addTriToVB(D3DXVECTOR3 pos,
		FLOAT r, FLOAT g, FLOAT b, FLOAT a);

	/**
	* This method is just responsible for signaling to the vertex buffer
	* that we wish to flush out the memory area of any pending triangle
	* data
	* @param void
	* @return void
	*/
	void flushVB(void);

	/**
	* (v1.3)
	* overloaded the addTriToVB method to account for texture mapping coordinates
	* @param D3DXVECTOR3 - (x, y, z) position of our vertex
    * @param FLOAT       - red component of our vertex
	* @param FLOAT       - green component of our vertex
	* @param FLOAT       - blue component of our vertex
	* @param FLOAT       - alpha component of our vertex
	* @param D3DXVECTOR2 - (u, v) texture mapping coordinates
	* @return void
	*/
	void addTriToVB(D3DXVECTOR3, FLOAT, FLOAT, FLOAT, FLOAT, D3DXVECTOR2);



};

#endif