
#ifndef __RENDERERINTERFACE_H_
#define __RENDERERINTERFACE_H_
/**************************************************************************************
* This file is part of the GameFramework project released by Wazoo Enterprises Inc.
* under the GNU Public License. 
*
* Any projects using this code must attach these copyright notices along with their
* own as outlined at http://www.gnu.org
*
* Copyright 2003 by Wazoo Enterprises Inc.
* To contact the auther, send a message to wazoo@wazooenterprises.com
*
****************************************************************************************/
#include <windows.h>
#include <string>

/**
* version 1.1
* Added our Vertex Buffer interface
**/
#include "vbInterface.h"

/**
* version 1.3
* Added our texture manager interface
**/
#include "textureManagerInterface.h"

/**
	*v1.4
	* Added states that can be enabled/disabled...note that this is not a complete list
	*/
	enum eState 
	{ 
		LIGHTING,         /** enable/disable our lighting */
		ZBUFFER,          /** enable/disable our zbuffer/depth buffer */
		ALPHABLENDING,    /** enable/disable our blending */
		SRC_BLEND,        /** set our primary alpha blending op */
		DEST_BLEND,       /** set our secondary alpha blending op */
		ALPHATEST,         /** enable/disable our alpha testing */
		STENCIL,				/** enable/disable our stencil buffer */
	};

	enum eOp 
	{ 
		ZERO, 
		ONE, 
		SRC, 
		SRC_ALPHA, 
		INV_SRC, 
		INV_SRC_ALPHA,
		DEST, 
		DEST_ALPHA, 
		INV_DEST, 
		INV_DEST_ALPHA, 
		ENABLE, 
		DISABLE
	};

	/** 
	* This enumeration defines the different buffers available to us, and which
	* one's pixel comparison function we wish to modify
	*/
	enum eFunction
	{
		ALPHATEST_FUNC, /** modify the alpha-test buffer comparison function */
		ZBUFFER_FUNC,   /** modify the zbuffer (Depth buffer) comparison function */
		STENCIL_FUNC    /** modify the stencil buffer comparion function */

	};

	/**
	* This enumeration defines our comparison function to perform on the pixel
	* of the state defined above
	*/
	enum eCompareOp
	{
		NEVER,		  /** never pass the test */
		LESS,		  /** accept the new pixel if the value < the current pixel */
		EQUAL,        /** accept the new pixel if the value = the current pixel */
		LESSEQUAL,    /** accept the new pixel if the value <= the current pixel */
		GREATER,      /** accept the new pixel if the value > the current pixel */
		NOTEQUAL,     /** accept the new pixel if the value != the current pixel */
		GREATEREQUAL, /** accept the new pixel if the value >= the current pixel */
		ALWAYS        /** always pass the comparison test */

	};

/**
* This class is our "base class" for obtaining
* some sort of GraphicsAPI independance. 
* We'll use this interface object to declare
* "high-level" methods which our "low-level" DLL
* projects will implement.
*/
class rendererInterface
{
protected:
	HWND m_hWnd;         /** handle to our main window */
	DWORD m_dwWidth;     /** our resolution width      */
	DWORD m_dwHeight;    /** our resolution height     */
	DWORD m_dwDepth;     /** our resolution depth      */
	BOOL m_bWindowed;    /** flag for being windowed or fullscreen */

	BOOL m_bUseStencilBit;  /** flag for clearing our stencil buffer */
	BOOL m_bUseDepthBit;    /** flag for clearing our depth buffer */

public:
	/**
	* v1.1
	* Added a pointer to our Vertex Buffer interface object
	*/
	vbInterface* m_pVB; /** pointer to our Vertex Buffer interface object */

	/**
	* v1.3
	* Added a pointer to our texture manager interface object
	*/
	textureManagerInterface* m_pTextureManager; /** pointer to our texture manager */

	
public:
	/**
	* Constructor
	*/
	rendererInterface(){};
	
	/**
	* Destructor
	*/
	virtual ~rendererInterface(){};

	/**
	* Method to create and initialize our hardware
	* @param HWND - our window handle
	* @param DWORD - our window width
	* @param DWORD - our window height
	* @param DWORD - our window depth
	* @param BOOL  - flag for being windowed or fullscreen
	* @return HRESULT - SUCCEEDED if we're ok, FAILED otherwise
	*/
    virtual HRESULT createDevice(HWND, DWORD, DWORD, DWORD, BOOL) = 0;

    /**
	* Method to destroy it
	* @param void
	* @return void
	*/
    virtual void destroyDevice(void) = 0;

    /**
	* Method to begin rendering primitives
	* @param void
	* @return HRESULT - SUCCEEDED if our hardware pipeline is ready, FAILED if not
	*/
    virtual HRESULT beginRenderingScene(void) = 0;

    /**
	* Method to end rendering primitives
	* @param void
	* @return void
	*/
    virtual void endRenderingScene(void) = 0;

    /**
	* Method to change the screen clearing color
	* @param FLOAT - red component
	* @param FLOAT - green component
	* @param FLOAT - blue component
	* @param FLOAT - alpha component
	* @return void
	*/
    virtual void setClearColor(FLOAT, FLOAT, FLOAT, FLOAT) = 0;

	/**
	* Method to return our pointer to our textureManagerInterface
	* @param void
	* @return textureManagerInterface* - pointer to our textureManagerInterface
	*/
	textureManagerInterface* getTextureInterface(void){ return m_pTextureManager;};

	/**
	* v1.4 
	* Method to enable/disable a state within the engine
	* @param eState - which state we're trying to modify
	* @param eOp     - the operation to perform on said state
	* @return void
	*/
	virtual void setState(const eState, const eOp) = 0;

	/**
	* v1.4
	* Method to adjust/define the comparison function for the desired buffer
	* @param eFunction - which buffer to modify
	* @param eCompareOp - which comparison method to use
	* @param DWORD      - our value to compare our pixel against
	* @return void
	*/
	virtual void setFunction(const eFunction, const eCompareOp, FLOAT fRef) = 0;

	/**
	* v1.4
	* Method to enable/disable clearing of the stencil buffer
	* @param BOOL - TRUE to enable stencil buffer clearing, FALSE to not
	* @return void
	*/
	void setStencilClearBit(BOOL bVal){ m_bUseStencilBit = bVal; };

	/**
	* v1.4
	* Method to enable/disable clearing of the depth buffer
	* @param BOOL - TRUE to enable depth buffer clearing, FALSE to not
	* @return void
	*/
	void setDepthClearBit(BOOL bVal){ m_bUseDepthBit = bVal;};

};

extern "C"
{
	HRESULT createRendererInterface(rendererInterface **pInterface);
	typedef HRESULT (*CREATERENDERERINTERFACE)(rendererInterface **pInterface);

	HRESULT destroyRendererInterface(rendererInterface **pInterface);
	typedef HRESULT (*DESTROYRENDERERINTERFACE)(rendererInterface **pInterface);
}

#endif