
#ifndef __VBINTERFACE_H_
#define __VBINTERFACE_H_
/**************************************************************************************
* This file is part of the GameFramework project released by Wazoo Enterprises Inc.
* under the GNU Public License. 
*
* Any projects using this code must attach these copyright notices along with their
* own as outlined at http://www.gnu.org
*
* Copyright 2003 by Wazoo Enterprises Inc.
* To contact the auther, send a message to wazoo@wazooenterprises.com
*
****************************************************************************************/


#include <windows.h>
#include <d3dx8.h>

/**
* This object is our interface to our Vertex Buffer object which
* is available in both OpenGL (via extensions) or through the ID3DVertexBuffer8
* interface available with DirectX8.0
*/
class vbInterface {

protected:


public:
	/**
	* Constructor
	*/
	vbInterface(){};

	/**
	* Destructor
	*/
	virtual ~vbInterface(){};

	/**
	* This method is responsible for creating our vertex buffer interface
	* object. Because OpenGL and Direct3D8 initialize things just a little
	* bit differently, I decided to give us the flexibility to pass in any
	* "extra" object that we would need to create our vertex buffer.
	* @param int - How many triangles our vertex buffer should hold
	* @param VOID* - A pointer to any extra creation object we need
	* @return HRESULT - SUCCEEDED if things are ok, FAILED otherwise
	*/
	virtual HRESULT createVB(int, VOID* = NULL) = 0;

	/**
	* This method just destroys our vertex buffer and handles any
	* memory cleanup
	* @param void
	* @return void
	*/
	virtual void destroyVB(void) = 0;

	/**
	* This method is responsible for locking down the vertex buffer
	* memory. Basically the hardware will just lock down an area that
	* we can use to add/update any triangle data.
	* @param void
	* @return HRESULT - if we SUCCEEDED or FAILED
	*/
	virtual HRESULT lockVB(void) = 0;

	/**
	* This method is responsible for signalling to the vertex buffer
	* that we are finished adding triangles. Let's unlock the video
	* memory, so that we can perform other operations with the 
	* hardware/pipeline
	* @param void
	* @return void
	*/
	virtual void unlockVB(void)  = 0;

	/**
	* This method is responsible for adding a triangle to our vertex
	* buffer interface. Although it IS a bit of a waste having a virtual
	* method being called for every single triangle that we add, we 
	* are just trying to create a way to demonstrate a fashion of rendering
	* primitives with both OpenGL and Direct3D8. The reader can optimize
	* as desired.
	* @param D3DXVECTOR3 - (x, y, z) position of our vertex
	* @param FLOAT       - red component of our vertex
	* @param FLOAT       - green component of our vertex
	* @param FLOAT       - blue component of our vertex
	* @param FLOAT       - alpha component of our vertex
	* @return void 
	*/
	virtual void addTriToVB(D3DXVECTOR3 pos,
		FLOAT r, FLOAT g, FLOAT b, FLOAT a) = 0;

	/**
	* This method is just responsible for signaling to the vertex buffer
	* that we wish to flush out the memory area of any pending triangle
	* data
	* @param void
	* @return void
	*/
	virtual void flushVB(VOID) = 0;

	/**
	* (v1.3)
	* overloaded the addTriToVB method to account for texture mapping coordinates
	* @param D3DXVECTOR3 - (x, y, z) position of our vertex
    * @param FLOAT       - red component of our vertex
	* @param FLOAT       - green component of our vertex
	* @param FLOAT       - blue component of our vertex
	* @param FLOAT       - alpha component of our vertex
	* @param D3DXVECTOR2 - (u, v) texture mapping coordinates
	* @return void
	*/
	virtual void addTriToVB(D3DXVECTOR3, FLOAT, FLOAT, FLOAT, FLOAT, D3DXVECTOR2) = 0;

};


#endif 